@@ -1,3 +1,28 @@
+# Copyright (c) 2018, Adrian Dusa
+# All rights reserved.
+# 
+# Redistribution and use in source and binary forms, with or without
+# modification, in whole or in part, are permitted provided that the
+# following conditions are met:
+#     * Redistributions of source code must retain the above copyright
+#       notice, this list of conditions and the following disclaimer.
+#     * Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
+#     * The names of its contributors may NOT be used to endorse or promote products
+#       derived from this software without specific prior written permission.
+# 
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
+# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 `findRows` <-
 function(expression = "", obj, remainders = TRUE, type = 1) {
     if (any(type == 0)) {
@@ -15,12 +40,18 @@ function(expression = "", obj, remainders = TRUE, type = 1) {
         noflevels <- obj$noflevels
         conditions <- obj$options$conditions
         if (any(is.element(type, c(0, 2, 3)))) {
-            call <- obj$call
-            call[[2]] <- obj$initial.data
-            if (!obj$options$neg.out) {
-                call[[3]] <- paste("~", call[[3]], sep = "")
+            call <- as.list(obj$call)[-1]
+            call$data <- obj$initial.data
+            if (obj$options$neg.out) {
+                if (tilde1st(call$outcome)) {
+                    call$outcome <- notilde(call$outcome)
+                }
             }
-            nobj <- suppressWarnings(eval(call))
+            else {
+                call$outcome <- paste("~", call$outcome, sep = "")
+            }
+            call$incl.cut <- rev(obj$options$incl.cut)
+            nobj <- suppressWarnings(do.call("truthTable", call))
         }
     }
     else {
